/* A lot of the operations defined here split up into various sub-tasks.
   This makes it work in a cooperative multithreading sort of way, but makes the
   code quit hard to follow.  This could use some better organization and
   documentation.
*/

var prevAlbum = "";
var artworkIsRemote;

var artworkThrobAnimation = new DDTintAnimator([180, 0, 0], [90, 60, 20],
                                               3000, true, [artworkThrobber]);
artworkThrobAnimation.fps = 10; // Default 20fps is way more than enough.
artworkThrobAnimation.onAnimationEnd = function () {
    artworkThrobber.hslTinting = "";
}

var artworkFadeAnimation = new DDFadeAnimator(0, 255, FADE_IN_TIME, false,
                                              [artwork]);
artworkFadeAnimation.fps = 10;
artworkFadeAnimation.onAnimationEnd = function () {
    artworkStatus.data = "";
    updateNow();
    if (artworkThrobAnimation.running)
        artworkThrobAnimation.stop();
    if (artworkIsRemote && 
          preferences.autoExportArtwork.value==true &&
          currentTrack.album != "")
        exportArtwork();
}

function artworkEnabled() {
    return (preferences.showArtwork.value == "true") ||
           (preferences.showArtwork.value == 1);
}

function toggleArtwork() {
    if (artworkEnabled())
        preferences.showArtwork.value = 0;
    else
        preferences.showArtwork.value = 1;        
    getArtwork(true);
}

function getArtwork (force) {
    if (force)
        prevAlbum = "";
    var album = iTunes.trackArtist + " - " + iTunes.trackAlbum;
    if (!artworkEnabled()) {
        clearArtwork();
        artwork_window.visible = 0;
        return;
    }
    else if (preferences.autoHideArtworkWindow.value == 0 &&
             artwork_window.visible == 0)
        artwork_window.visible = 1;
    if (album != prevAlbum) {
        prevAlbum = album;
        artworkStatus.data = "Loading image…";
        if (artwork.opacity != 0)
            artwork.fade(artwork.opacity, 0, 2);
        if (!getArtworkFromFile(force)) {
            getArtworkFromAmazon(album, 0);
            return false;
        }
        else
            return true;
    }
    return false;
}

function getArtworkFromFile (force) {
    var imgSrc;
    if (system.platform == "macintosh")
        imgSrc = appleScript("Scripts/Get_Artwork.applescript");
    else {
        var iTunesApp = iTunesCOM();
        if (iTunesApp.CurrentTrack.Artwork.Count > 0) {
            imgSrc = system.temporaryFolder + "/iTunesBar-artwork.pict";
            iTunesApp.CurrentTrack.Artwork.Item(1).SaveArtworkToFile(convertPathToPlatform(imgSrc));
        }
        else
            imgSrc = "none";
    }
        
    if (imgSrc == "none")
        return false;
    else {
        setArtwork(imgSrc, false);
        return true;
    }
}

function setArtwork(src, remote) {
    if (src == "") {
        clearArtwork();
        return;
    }
    artworkIsRemote = remote;
    artworkStatus.data = "Loading image…";
    artwork.src = src;
    artwork.reload();
    if (artwork_window.visible == 0)
        artwork_window.visible = 1;
    artworkFadeAnimation.start();
}

function clearArtwork (noReset) {
    if (artworkThrobAnimation.running)
        artworkThrobAnimation.stop();
    if (preferences.autoHideArtworkWindow.value == 1) {
        artwork_window.visible = 0;
        artwork.opacity = 0;
    }
    else
        artwork.fade(artwork.opacity, 0, 3);
    if (!noReset)
        prevAlbum = "";
}

// E.g. http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=184H2A27EEDJ528WS802&Operation=ItemSearch&ResponseGroup=Images&SearchIndex=Music&Keywords=Arcade%20Fire
var AWS_BASES = [{name: "Amazon USA",
                  url: "http://webservices.amazon.com/onca/xml"},
                 {name: "Amazon Japan",
                  url: "http://webservices.amazon.co.jp/onca/xml"} ];
var AWS_HEADERS = { Service: "AWSECommerceService",
                    SubscriptionId: "184H2A27EEDJ528WS802",
                    Operation: "ItemSearch",
                    ResponseGroup: "Images",
                    SearchIndex: "Music" }
var AWS_REQUEST;

function getArtworkFromAmazon(searchStr, attempt) {
    if (AWS_REQUEST)
        AWS_REQUEST.abort();
    artworkStatus.data = "Querying " + AWS_BASES[attempt].name + "…";
    var req = new XMLHttpRequest();
    var url = AWS_BASES[attempt].url + "?";
    for (var i in AWS_HEADERS) {
        url += i + "=" + escape(AWS_HEADERS[i]) + "&";
    }
    url += "Keywords=" + escape(searchStr);
    req.open("GET", url, true);
    req.onreadystatechange = handleAmazonResponse;
    req.attempt = attempt;
    req.searchStr = searchStr;
    req.send();
    
    AWS_REQUEST = req;
    artworkThrobAnimation.start();
}    

function handleAmazonResponse() {
    if (this.readyState != 4)
        return;
    if (this.status != 200) {
        artworkFailed(this.statusText == "" ? this.responseText :
                                              this.statusText,
                      this);
        return;
    }
    
    var doc = this.responseXML;
    var items = doc.evaluate("ItemSearchResponse/Items/Item");
    var item = items.item(0);
    var image = "";
    if (item) {
        image = item.evaluate("string(LargeImage/URL)");
        if (image == "")
            image = item.evaluate("string(MediumImage/URL)");
        if (image == "")
            image = item.evaluate("string(SmallImage/URL)");
    }
    if (image != "") {
        downloadAmazonArtwork(image);
    }
    else {
        artworkFailed("No artwork found ", this);
    }
}

function downloadAmazonArtwork(imgURL) {
    artworkStatus.data = "Loading image…";
    var url = new URL();    
    url.location = imgURL;
    url.outputFile = system.temporaryFolder + "/iTunesBar-artwork" + getFileExtension(imgURL);
    //print("Fetching image " + url.location + " to " + url.outputFile);
    url.fetchAsync(downloadedAmazonArtwork);
}

function downloadedAmazonArtwork(url) {
    print("Fetching image response: " + url.response);
    if (url.response == 200) {
        setArtwork(url.outputFile, true);
    }
    else {
        artworkFailed(url.responseData);
    }
}

// remoteArtworkLoaded is called when the artwork src is loaded.
//function remoteArtworkLoaded() {
//    artworkStatus.data = "";
//    artworkThrobAnimation.stop();
//    artworkFadeAnimation.start();
//}

function artworkFailed(msg, req) {
    if (req && req.attempt < (AWS_BASES.length - 1)) {
        getArtworkFromAmazon(req.searchStr, req.attempt + 1);
    }
    else {
        artworkThrobAnimation.stop();
        artworkStatus.data = msg;
        clearArtwork(true);
    }
}

function copyArtwork() {
    var outfile = saveAs();
    if (!outfile)
        return;
    saveImageToFile(artwork, outfile);
}

var EXT_REGEX = /\.(jpg|jpeg|png|gif|bmp|pict)$/i;
function getFileExtension(filename) {
    return filename.match(EXT_REGEX)[0];
}

function saveImageToFile(img, outfile) {
    var infile = (img.src == img.src[0] ? img.src[0] : img.src);
    if (infile.indexOf("http://") == 0) {
        // img is remote URL
        var filename = outfile + getFileExtension(infile);
        var url = new URL();
        url.location = infile;
        url.outputFile = filename;
        print("Fetching URL " + url.location + " to " + url.outputFile);
        url.fetch();
        print(url.response);
    }
    else {
        // artwork is local
        var filename = outfile + getFileExtension(infile);
        filesystem.copy(infile, filename);
    }
    return filename;
}

if (system.platform == "macintosh") {
    function exportArtwork() {
        if (artwork.src == "blank.png") {
            return;
        }
        if (!currentTrack.artist.isPrintableAscii() ||
            !currentTrack.album.isPrintableAscii()) {
            return;
        }
            
        var trackInfoData = skinElements.trackInfo.data;
        skinElements.trackInfo.data = "Embedding Artwork... " + trackInfoData;
        updateNow();
        var imgFile = artwork.src;
        var pictFile = system.temporaryFolder + "/iTunesBar-artwork.pict";
        print("Embedding artwork for " + currentTrack.artist + "/" + currentTrack.album);
        appleScript("tell application \"Image Events\" \n" +
                    "  set theImg to open \"" + imgFile + "\" \n" +
                    "  save theImg as PICT in \"" + pictFile + "\" \n" +
                    "  close theImg \n" +
                    "end tell \n", 60);
        sleep(500);
        var result = telliTunes(
          "set theFile to open for access (posix file \"" + pictFile + "\") \n" +
          "try \n" +
          "  set theData to (read theFile from 513 as picture) \n" +
          "  close access theFile \n" +
          "on error e \n" +
          "  close access theFile \n" +
          "  error e \n" +
          "end try \n" + 
          "tell the first library playlist \n" +
          "  set theTracks to every track whose (artist is \"" + currentTrack.artist + "\" and album is \"" + currentTrack.album + "\") \n" +
          "end tell \n" +
          "repeat with tr in theTracks \n" +
          "  tell tr to set the data of front artwork to theData \n" +
          "end repeat \n", 180);
        if (result == "error") {
            alert("Embedding album artwork for \"" + currentTrack.album + "\" by " + currentTrack.artist + " failed.");
        }
        skinElements.trackInfo.data = trackInfoData;
    }
}
else { // system.platform == "windows"
    function exportArtwork() {
        if (artwork.src == "blank.png")
            return;
            
        var trackInfoData = skinElements.trackInfo.data;
        skinElements.trackInfo.data = "Embedding Artwork... " + trackInfoData;
        updateNow();
        imgFile = artwork.src;
        print("Embedding artwork for " + currentTrack.artist + "/" + currentTrack.album + " (" + imgFile + ")");
        var tracks = iTunesCOM().LibraryPlaylist.Search(currentTrack.album, 3);
        if (tracks) {
            for (var i = 1; i <= tracks.Count; i++) {
                var track = tracks.Item(i);
                if (track.Artist == currentTrack.artist) {
                    if (track.Artwork.Count == 0) {
                        print("Adding artwork to " + track.Name);
                        track.AddArtworkFromFile(imgFile);
                    }
                    else {
                        print("Replacing artwork of " + track.Name);
                        track.Artwork.Item(1).SetArtworkFromFile(imgFile);
                    }
                }
            }
        }
        skinElements.trackInfo.data = trackInfoData;
    }
}

function embedArtworkInFile() {
    if (alert("This action will set the default artwork of all tracks in the album \"" + currentTrack.album + "\" by " + currentTrack.artist + " to the image currently displayed, replacing any existing default images.\n\nThis operation is irreversible.", "Continue", "Cancel") == 1) {
        exportArtwork();
    }
}

