function makeMenuItem(title, onSelect, enabled, checked) {
    var item = new MenuItem();
    item.title = title;
    item.onSelect = onSelect;
    if (typeof checked != "undefined")
        item.checked = checked;
    if (typeof enabled != "undefined")
        item.enabled = enabled;
    return item;
}

function showPopupMenu() {
    popupMenu(popupMenuItems(), skinElements.iTunesPopup.hOffset, skinElements.iTunesPopup.vOffset);
}

function createMainContextMenu() {
    main_window.contextMenuItems = popupMenuItems();
}

function createArtworkContextMenu() {
    var items = new Array();
    items.push(makeMenuItem("Close Artwork Window",
                            "preferences.showArtwork.value = 0;\n" +
                            "getArtwork();\n"));
    items.push(makeMenuItem("Save Artwork As…", copyArtwork,
                            artwork.opacity == 255));
    items.push(makeMenuItem("Embed Artwork in Album Song Files…",
                            embedArtworkInFile,
                            artwork.opacity == 255));
    artwork_window.contextMenuItems = items;
}

function popupMenuItems() {
    var items = new Array();
    
    // Skin selector...
    items.push(makeMenuItem("Skin Manager…",
                            "buildSkinGroupList(); skin_window.visible = 1;"))
    
    // Artwork and lyrics searching...
    items.push(makeMenuItem("Show Artwork", "toggleArtwork()", 
                            true, artworkEnabled()));
    
    if (system.event.modifiers.indexOf("option") >= 0 ||
            system.event.modifiers.indexOf("alt") >= 0)
        items.push(makeMenuItem("Search the Fake Book Index…", "lookupFakeBook"));
    else
        items.push(makeMenuItem("Search for Lyrics…", "lookupLyrics()"))
    
    // Shuffle and repeat modes...
    items.push(menuSeparator);
    var shuffle = iTunes.running ? iTunes.shuffle : false;
    items.push(makeMenuItem("Shuffle Songs", "iTunes.shuffle = !iTunes.shuffle",
                            iTunes.running, shuffle));
    
    items.push(menuSeparator);
    var repeat = iTunes.running ? iTunes.repeatMode : "N/A";
    items.push(makeMenuItem("No Repeat", "iTunes.repeatMode = 'off'",
                            iTunes.running, repeat=="off"));
    items.push(makeMenuItem("Repeat All", "iTunes.repeatMode = 'all'",
                            iTunes.running, repeat=="all"));
    items.push(makeMenuItem("Repeat One", "iTunes.repeatMode = 'one'",
                            iTunes.running, repeat=="one"));
    
    // Playlists...
    items = items.concat(getPlaylistMenuItems());
    
    // iTunes interaction...
    items.push(menuSeparator);
    if (manualSuspend)
        items.push(makeMenuItem("Resume iTunes Bar", "resume()"));
    else
        items.push(makeMenuItem("Suspend iTunes Bar",
                                "suspend('iTunes Bar suspended.  Click the iTunes button to resume.', true)"));
    items.push(makeMenuItem("Open iTunes", "openiTunes()"));
    items.push(makeMenuItem("Quit iTunes", "quitiTunes()", iTunes.running));
    
    return items;
}

function getPlaylistMenuItems() {
    var items = new Array();
    if (iTunes.running) {
        items.push(menuSeparator);
        items.push(makeMenuItem("Playlists", "", false));
        if (system.platform == "macintosh") {            
            var playlists = telliTunes("return the name of every playlist as text");
            var current = telliTunes("return the name of the current playlist");
            if (current == "error")
                current = telliTunes("return the name of the view of the front browser window");
            playlists = playlists.split("\r");
            function makeEventHandler (playlist) {
                return function () {
                    telliTunes("set p to the first playlist whose name is \"" + playlist + "\"\n" +
                               "set the view of the front browser window to p\n" +
                               "tell p to play");
                }
            }
            for (var i in playlists) {
                var pl = playlists[i];
                items.push(makeMenuItem("  " + pl, makeEventHandler(pl),
                                        true, pl == current));
            }
        }
        else { // system.platform == "windows"
            var com = iTunesCOM();
            var playlists = com.LibrarySource.Playlists;
            var current = com.CurrentPlaylist;
            current = current ? current.Name : "none";
            function makeEventHandler (playlist) {
                return function () {
                    print("Setting playlist to " + playlist.Name);
                    playlist.PlayFirstTrack();
                    // This next statement doesn't seem to work :(
                    com.BrowserWindow.SelectedPlaylist = playlist;
                }
            }
            for (var i = 1; i <= playlists.Count; i++) {
                var pl = playlists.Item(i);
                items.push(makeMenuItem("  " + pl.Name, makeEventHandler(pl),
                                        true, pl.Name == current));
            }
        }
    }
    return items;
}