


/******************* Skins Directory Init ***************/

var skinsFolder = system.widgetDataFolder + "/Skins";
    
function initSkinsDirectory() {
	if ((preferences.iTunesBarVersion.value != widget.version
            || preferences.installSkins.value == 1)
        && alert("New and updated iTunes Bar skins for version " + 
                 widget.version + " will be installed.\n\n" +
                 "Old versions of the skins will be overwritten " + 
                 "(other skins will be left alone).  If you cancel, " +
                 "you may install them later by selecting \"Install " +
                 "Default Skins\" in the preferences.\n\n" +
                 "(Currently installed version: " +
                    (preferences.iTunesBarVersion.value<0 ? "none" : 
                                    preferences.iTunesBarVersion.value) + ")", 
                 "Install Skins", "Cancel") == 1)
	{
		print(	"Installing default skins for version " + widget.version +
				".  (Previously installed version: " +
				preferences.iTunesBarVersion.value + ")");
			
        if (!filesystem.itemExists(skinsFolder))
        {
            filesystem.createDirectory(skinsFolder);
        }
        
        var errorSkins = new Array();
        var groupList = filesystem.getDirectoryContents("Resources/Skins/", false);
        for (var i in groupList)
        {
            var fromGroup = "Resources/Skins/" + groupList[i];
            var toGroup = skinsFolder + "/" + groupList[i];
            
            if (filesystem.isDirectory(fromGroup))
            {
                if (!filesystem.itemExists(toGroup))
                {
                    filesystem.createDirectory(toGroup);
                }
                
                var skinList = filesystem.getDirectoryContents(fromGroup, false);
                for (var j in skinList)
                {
                    var fromSkin = fromGroup + "/" + skinList[j];
                    var toSkin = toGroup + "/" + skinList[j];
                    if (filesystem.itemExists(toSkin)
                        && !filesystem.remove(toSkin))
                    {
                        print("Could not remove old skin " + toSkin);
                    }
                    if (!filesystem.copy(fromSkin, toSkin))
                    {
                        errorSkins.push(skinList[j]);
                    }
                }
            }
            else // For readme file etc.
            {
                if (filesystem.itemExists(toGroup)
                    && !filesystem.remove(toGroup))
                {
                    print("Could not delete file " + toGroup);
                }
                filesystem.copy(fromGroup, toGroup);
            }
        }
        if (errorSkins.length > 0)
        {
            alert("Error installing default skins.  The following skins were not installed:\n    " +
                  errorSkins.join("\n    ") + "\nPlease contact danieldickison@cmu.edu if the problem persists.");
        }
    }
    
	preferences.installSkins.value = 0;
}



/******************** Skin Loading ************************/

// Cache of currently loaded skin for faster resize drawing.
var skinDOMCache = null;

var skinElements = {
    bg_left: null, 
    bg_center: null,
    bg_right: null,
    reflection_left: null, 
    reflection_center: null, 
    reflection_right: null,
    volumeUp: null, 
    volumeDown: null, 
    play: null, 
    pause: null, 
    reverse: null, 
    forward: null,
    iTunesPopup: null,
    resizeThumb: null,
    trackInfo: null,
    trackInfo_shadow: null,
    trackTime: null,
    trackTime_shadow: null,
    transport_frame: null, 
    transport_dot: null,
    ratingStars: null,
    volumeIndicator: null
}
var DEFAULT_BG_Z = 0;
var DEFAULT_REFL_Z = 10;
var DEFAULT_BUTTONS_Z = 5;


function loadSkin(skinDir) {
    try {
        var file = skinDir + "/Skin Info.xml";
        print("Reading " + file + "...");
        var xml = filesystem.readFile(file);
        var doc = XMLDOM.parse(xml); 
        var dom = doc.getElementsByName("skin").item(0);
        setupSkin(dom, skinDir, true);
        skinDOMCache = dom;
        resume();
    }
    catch (e) {
        alert("Error while loading the skin \"" + skinDir + "\".  Reverting to the default skin.\n\n" + 
              e.toString());
        preferences.skinPath.value = preferences.skinPath.defaultValue;
        loadSkin(preferences.skinPath.value);
    }
}

function rebuildMainGUI() {
    var root = main_window.root;
    print("Removing current skin elements...");
    // Remove all the subviews...
    for (var i in skinElements) {
        if (skinElements[i] && skinElements[i].removeFromSuperview)
            skinElements[i].removeFromSuperview();
    }
    
    // Some utility functions...
    function addText() {
        var txt = new Text();
        txt.zOrder = DEFAULT_BUTTONS_Z;
        root.addSubview(txt);
        return txt;
    }
    function addImage(zOrder, mDown, mUp, mEnter, mExit, mDrag) {
        var img = new Image();
        img.src = "Resources/blank.png";
        img.zOrder = zOrder;
        if (mDown)
            img.onMouseDown = mDown;
        if (mUp)
            img.onMouseUp = mUp;
        if (mEnter)
            img.onMouseEnter = mEnter;
        if (mExit)
            img.onMouseExit = mExit;
        if (mDrag)
            img.onMouseDrag = mDrag;
        root.addSubview(img);
        return img;
    }
    function addButton(action, initAction, enterAction, exitAction) {
        var button = addImage(DEFAULT_BUTTONS_Z, 
                              _button_mouseDown, _button_mouseUp,
                              _button_mouseEnter, _button_mouseExit);
        button.action = action;
        if (initAction)
            button.initAction = initAction;
        if (enterAction)
            button.enterAction = enterAction;
        if (exitAction)
            button.exitAction = exitAction;
        return button;
    }
    function addScrub(action, initAction, endAction) {
        var scrub = addImage(DEFAULT_BUTTONS_Z,
                             _scrub_mouseDown, _scrub_mouseUp, 
                             null, null, _scrub_mouseMove);
        scrub.action = action;
        if (initAction)
            scrub.initAction = initAction;
        if (endAction)
            scrub.endAction = endAction;
        return scrub;
    }
    
    // Now add all the crap...
    print("Creating new skin elements...");
    skinElements.bg_left = addImage(DEFAULT_BG_Z);
    skinElements.bg_center = addImage(DEFAULT_BG_Z);
    skinElements.bg_right = addImage(DEFAULT_BG_Z);
    skinElements.reflection_left = addImage(DEFAULT_REFL_Z);
    skinElements.reflection_center = addImage(DEFAULT_REFL_Z);
    skinElements.reflection_right = addImage(DEFAULT_REFL_Z);
    skinElements.volumeUp = addButton(_volumeUp_action);
    skinElements.volumeDown = addButton(_volumeDown_action);
    skinElements.play = addButton(_playButton_action);
    skinElements.pause = addButton(_pauseButton_action);
    skinElements.reverse = addButton(_reverseButton_action, _reverseButton_initAction, _reverseButton_enterAction, _reverseButton_exitAction);
    skinElements.forward = addButton(_forwardButton_action, _forwardButton_initAction, _forwardButton_enterAction, _forwardButton_exitAction);
    skinElements.iTunesPopup = addImage(DEFAULT_BUTTONS_Z, 
                                        function () {
                                            this.highlight();
                                            showPopupMenu();
                                            this.unhighlight();
                                        });
    skinElements.resizeThumb = addImage(DEFAULT_BUTTONS_Z,
                                        _resizeInit, _resizeDone,
                                        null, null, _resizeMove);
    skinElements.resizeThumb.doResize = _main_resize;
    skinElements.resizeThumb.resizeWindow = main_window;
    skinElements.trackInfo = addText();
    skinElements.trackInfo.onMultiClick = function () {
                                              this.color = this.activeColor;
                                              updateNow();
                                              trackInfoClick();
                                              this.color = this.normalColor;
                                          }
    skinElements.trackInfo_shadow = new Shadow();
    skinElements.trackTime = addText();
    skinElements.trackTime.onMouseDown = changeTimeDisplay;
    skinElements.trackTime_shadow = new Shadow();
    skinElements.transport_frame = addScrub(_positionFrame_action,
                                            _positionFrame_initAction,
                                            _positionFrame_endAction);
    skinElements.transport_dot = addImage(DEFAULT_BUTTONS_Z);
    skinElements.ratingStars = addScrub(_ratingStars_action);
    skinElements.volumeIndicator = addScrub(_volumeIndicator_action);
    
    setupGlobals();
}

function setupGlobals() {
    with (skinElements) {
        trackFadeAnimator.objects = [trackInfo, trackTime, transport_frame, transport_dot, ratingStars];
        suspendDimItems = [trackInfo, trackTime, volumeUp, volumeDown, volumeIndicator, play, transport_frame, reverse, forward];
    }
    currentTrack.clear();
}


function setupSkin(dom, baseDir, force) {
    if (force) {
        rebuildMainGUI();
    }
    
    main_window.minWidth = getMinWidth(dom, baseDir, force);
    main_window.width = Math.max(preferences.windowWidth.value,
                                 main_window.minWidth);
    main_window.height = dom.evaluate("number(window/@height)");
    
    var info = "";
    var attrs = dom.evaluate("info/@*");
    for (var i = 0; i < attrs.length; i++) {
        info += attrs.item(i).name + ":";
        for (var j = 0; j < 15 - attrs.item(i).name.length; j++)
            info += " ";
        info += "\t" + attrs.item(i).value + "\n";
    }
    skinInfo.data = skinInfo_decoration.data = info;
    
    for (var elem in skinElements) {
        var node = dom.getElementsByTagName(elem).item(0);
        if (node) {
            attrs = node.attributes;
            for (var i = 0; i < attrs.length; i++) {
                var attr = attrs.item(i);
                var item = translatePropertyValue(attr.name, attr.value,
                                                  node, baseDir);
                if (item) {
                    skinElements[elem][item.property] = item.value;
                }
            }
        }
    }
    if (dom.getElementsByTagName("trackInfo_shadow").length > 0) {
        skinElements.trackInfo.shadow = skinElements.trackInfo_shadow;
    }
    if (dom.getElementsByTagName("trackTime_shadow").length > 0) {
        skinElements.trackTime.shadow = skinElements.trackTime_shadow;
    }
    
    // Make visual elements take on the new looks...
    if (force) {
        for (elem in skinElements) {
            if (skinElements[elem].srcPrefix) {
                skinElements[elem].src = skinElements[elem].srcPrefix + "0" + skinElements[elem].srcExtension;
                skinElements[elem].reload();
            }
            else if (skinElements[elem].src) {
                skinElements[elem].normalSrc = skinElements[elem].src;
                skinElements[elem].reload();
            }
            else if (skinElements[elem].color)
                skinElements[elem].normalColor = skinElements[elem].color;
        }
    }
    
    // Force scrolling to recalc width...
    var temp = skinElements.trackInfo.scrolling;
    skinElements.trackInfo.scrolling = "off";
    skinElements.trackInfo.scrolling = temp;
    tweakTrackInfoLayout(dom.evaluate("number(ratingStars/@left)") >= 0);
    if (!main_window.visible)
        main_window.visible = 1;
}

function translatePropertyValue(name, value, node, baseDir) {
    if (name.indexOf("on") == 0 || name == "contextMenuItems") {
        alert("iTunes Bar skins are not permitted to specify event handling attributes.  The violating attribute has been ignored.\n\nNode: " + node.nodeName + "\nAttribute: " + name + "\nValue: \""+ value + "\"");
        return null;
    }
    switch (name) {
        case "left":
            return { property: "hOffset",
                     value: wrapAround(value, main_window.width) };
        case "right":
            return { property: "width",
                     value: main_window.width - 
                              Number(node.getAttribute("left")) - value };
        case "top":
            return { property: "vOffset",
                     value: Number(value) };
        case "bottom":
            return { property: "height",
                     value: main_window.height - 
                              Number(node.getAttribute("top")) - value };
        case "src":
            return { property: "src",
                     value: (value == null) ? "Resources/blank.png" :
                                              baseDir + "/" + value };
        case "activeSrc":
        case "inactiveSrc":
            return { property: name,
                     value: (value == null) ? null :
                                              baseDir + "/" + value };
        case "srcPrefix":
            return { property: "srcPrefix",
                     value: baseDir + "/" + value };
        //case "color":  -- this gets set later.
        //    return { property: "normalColor",
        //             value: value };
        case "dotOffset":
        case "movementRange":
        case "hOffset":
        case "vOffset":
        case "opacity":
        case "numStates":
        case "zOrder":
        case "size":
        case "height":
        case "width":
        case "minWidth":
            return { property: name,
                     value: Number(value) };
        default:
            return { property: name,
                     value: value };
    }
}

function getMinWidth(dom, baseDir, force) {
    if (force) {
        var newPre = dom.evaluate("string(ratingStars/@srcPrefix)");
        var newSuf = dom.evaluate("string(ratingStars/@srcExtension)");
        skinElements.ratingStars.src = baseDir + "/" + newPre + 0 + newSuf;
    }
    return dom.evaluate("number(window/@minWidth)") -
            (preferences.displayRating.value == 1 ?
                            0 : skinElements.ratingStars.srcWidth);
}

function tweakTrackInfoLayout(starsOnLeft) {
    with (skinElements) {
        if (preferences.displayRating.value == 0 &&
            ratingStars.vOffset > trackInfo.vOffset - trackInfo.size &&
            ratingStars.vOffset < trackInfo.vOffset + ratingStars.srcHeight)
        {
          trackInfo.width = trackInfo.width + ratingStars.srcWidth;
          if (starsOnLeft)
            trackInfo.hOffset = trackInfo.hOffset - ratingStars.srcWidth;
        }
        if (trackInfo.width <= 0)
            trackInfo.opacity = 0;
        else if (!suspended)
            trackInfo.opacity = 255;
        //print("to: " + trackInfo.hOffset + ", " + trackInfo.width);
    }
}

function wrapAround(value, limit) {
    value = Number(value);
    return (value < 0) ? limit + value : value;
}
