


/*********************** GUI *************************/

function createGUI(prefUpdate) {
    if (prefUpdate || skinDOMCache == null) {
        loadSkin(preferences.skinPath.value);
        setupWindowProperties();
        setupTrackInfoEvents();
        setupArtworkWindow();
            
        skinElements.trackInfo.separator = " " + preferences.separator.value + " ";
        skinElements.ratingStars.visible = preferences.displayRating.value;
            
        update();
    }
    else {
        setupSkin(skinDOMCache, preferences.skinPath.value)
    }
}

function setupArtworkWindow() {
    artwork_window.shadow = preferences.artworkWindowShadow.value;
    var size = preferences.artworkSize.value;
    if (size < 50)
        size = preferences.artworkSize.value = 50;
    artwork_window.width = size;
    artwork_window.height = size;
    artworkBg.width = size;
    artworkBg.height = size;
    artwork.width = size;
    artwork.height = size;
    artworkThrobber.width = artworkThrobber.srcWidth * size / artworkBg.srcWidth;
    artworkThrobber.height = artworkThrobber.srcHeight * size / artworkBg.srcHeight;
    artworkThrobber.hOffset = size * 0.22;
    artworkThrobber.vOffset = size * 0.19;
    artworkStatus.hOffset = size / 2;
    artworkStatus.vOffset = size * 0.9
    art_resize.hOffset = size - 18;
    art_resize.vOffset = size - 18;
    if (size > 400)
        artworkStatus.size = 12;
    else if (size < 100)
        artworkStatus.size = 6;
    else if (size < 150)
        artworkStatus.size = 8;
    else
        artworkStatus.size = 10;
    if (artworkEnabled() &&
          preferences.autoHideArtworkWindow.value == 0 &&
          artwork_window.visible != 1)
        artwork_window.visible = 1;
}



/********** Volume, Ratings, Hotkeys, TrackInfo and such **********/

function updateVolIndicators() {
    with (skinElements) {
        if (volumeIndicator.numStates != 0) {
            var v =  Math.round(currentStatus.volume * (volumeIndicator.numStates-1) / 100);
            volumeIndicator.src = volumeIndicator.srcPrefix + v + volumeIndicator.srcExtension;
        }
    }
}

function showRating(rating) {
    with (skinElements) {
        rating = Math.floor(rating / 20);
        ratingStars.src = ratingStars.srcPrefix + rating + ratingStars.srcExtension;
    }
}


function setupWindowProperties() {
    main_window.level = preferences.windowLevel.value;
    main_window.locked = preferences.windowLocked.value;
    main_window.opacity = preferences.windowOpacity.value;
    
    artwork_window.level = preferences.artworkWindowLevel.value;
    artwork_window.locked = preferences.artworkWindowLocked.value;
    artwork_window.opacity = preferences.artworkWindowOpacity.value;
}

function setupTrackInfoEvents() {
    with (skinElements) {
        trackInfo.inactive = false;
        trackInfo.onMouseEnter = "";
        trackInfo.onMouseExit = "";
        if ( preferences.alwaysScrollText.value == '1')
            trackInfo.scrolling = "autoLeft";
        else {
            trackInfo.scrolling = "off";
            trackInfo.onMouseEnter = function () {
                                        this.scrolling = "autoLeft";
                                     };
            trackInfo.onMouseExit = function () {
                                        this.scrolling = "off";
                                    }
        }
    }
}

function changeTimeDisplay() {
    switch (preferences.displayTimeType.value) {
        case "Total":
            preferences.displayTimeType.value = "Elapsed";
            break;
        case "Elapsed":
            preferences.displayTimeType.value = "Remaining";
            break;
        case "Remaining":
            preferences.displayTimeType.value = "Total";
            break;
        case "Off":
        default:
            preferences.displayTimeType.value = "Total";
            break;
    }
    setTrackTime();
}

function trackInfoClick() {
    if (system.event.modifiers == "shift")
        lookupFakeBook();
    else
        lookupLyrics();
}




var hotkeys = { play: playHK, back: backHK, next: nextHK, 
                volUp: volUpHK, volDown: volDownHK, iTunes: iTunesHK};
var hotkeyPrefs = { play: preferences.playHotKey,
                    back: preferences.backHotKey,
                    next: preferences.nextHotKey,
                    volUp: preferences.volUpHotKey,
                    volDown: preferences.volDownHotKey,
                    iTunes: preferences.iTunesHotKey };
var hotkeyActions = { play:     function () {
                                    if (iTunes.running) {
                                        iTunes.playPause(); 
                                        update();
                                    }
                                },
                      back:     function () {
                                    if (iTunes.running) {
                                        iTunes.backTrack(); 
                                        update();
                                    }
                                },
                      next:     function () {
                                    if (iTunes.running) {
                                        iTunes.nextTrack(); 
                                        update();
                                    }
                                },
                      volUp: "changeVolume(11);",
                      volDown: "changeVolume(-11);",
                      iTunes: openiTunes };

function setupHotkeys() {
	if ( preferences.hotkeysEnabled.value == 0) {
		log("Disabling hotkeys");
		for (var i in hotkeys) {
		    hotkeys[i].onKeyDown = "";
        }
	}
	else {
		log("Enabling hotkeys");
		var errors = "";
		for (var i in hotkeys) {
            var pref = hotkeyPrefs[i].value;
            print(i + ": " + pref);
            try {
                hotkeys[i].key = pref.substr(pref.lastIndexOf('+') + 1);
                hotkeys[i].modifier = pref.substr(0, pref.lastIndexOf('+'));
                hotkeys[i].onKeyDown = hotkeyActions[i];
            }
            catch (e) {
                errors += "\n" + i + ": " + e.toString();
            }
        }
        if (errors != "" &&
            prompt("One or more hotkeys failed to register.  These key combinations may be in use by other applications.  Please change them in the preferences.\n" + errors, "Ignore", "Open Prefs…") == 2) {
            showWidgetPreferences();
        }
	}
}


