/* Konfabulator Animation Voodoo
   Provides asynchronous animation capabilities.
   Author: Daniel Dickison (danieldickison@cmu.edu)
   Requires Konfabulator version 2+
   April 10, 2005
*/

var _animators = [];

function Animator() {
    this.id = _animators.length;
    _animators[this.id] = this;
    this.timer = new Timer();
    this.timer.interval = 0.05;
    this.progress = 0;
}

Animator.prototype.fps getter = function () {
    return 1.0 / this.timer.interval;
}
Animator.prototype.fps setter = function (fps) {
    this.timer.interval = 1.0 / fps;
}
Animator.prototype.ticking getter = function () {
    return this.timer.ticking;
}
Animator.prototype.ticking setter = function (tick) {
    if (tick)
        this.start();
    else
        this.stop();
}

Animator.prototype.objects = [];

var _currentAnim;
var _currentVal;
var _currentElapsed;
var _currentEase;

Animator.prototype.prepareFade = function (from, to, duration) {
    var delta = to - from;
    this.onTick = function () {
        _currentEase = 0.5 * (1 - Math.cos(Math.PI * this.progress));
        _currentVal = from + (delta * _currentEase);
        for (var i in this.objects)
            this.objects[i].opacity = _currentVal;
    }
    this._onAnimationEnd = function () {
        for (var i in this.objects)
            this.objects[i].opacity = to;
    }
    this.setupTimer(duration, duration);
}

Animator.prototype.prepareHSLThrob = function (from, to, period, duration) {
    var h0 = from[0];
    var s0 = from[1];
    var l0 = from[2];
    var dh = to[0] - h0;
    var ds = to[1] - s0;
    var dl = to[2] - l0;
    this.onTick = function () {
        _currentEase = 0.5 * (1 - Math.cos(2*Math.PI * this.progress));
        _currentVal = Math.floor(h0 + dh * _currentEase) + ", " +
                      Math.floor(s0 + ds * _currentEase) + ", " +
                      Math.floor(l0 + dl * _currentEase);
        for (var i in this.objects)
            this.objects[i].hslTinting = _currentVal;
    }
    this._onAnimationEnd = function () {
        for (var i in this.objects)
            this.objects[i].hslTinting = null;
    }
    this.setupTimer(period, duration);
}

Animator.prototype.prepareMove = function (x0, y0, x1, y1, duration) {
    var dx = x1 - x0;
    var dy = y1 - y0;
    this.onTick = function () {
        _currentEase = 0.5 * (1 - Math.cos(Math.PI * this.progress));
        _currentVal = [x0 + (dx * _currentEase), y0 + (dy * _currentEase)];
        for (var i in this.objects) {
            this.objects[i].hOffset = _currentVal[0];
            this.objects[i].vOffset = _currentVal[1];
        }
    }
    this._onAnimationEnd = function () {
        for (var i in this.objects)
            this.objects[i].hOffset = x1;
            this.objects[i].vOffset = y1;
    }
    this.setupTimer(duration, duration);
}

Animator.prototype.setupTimer = function (period, duration) {
    this.timer.onTimerFired = "_currentAnim = _animators[" + this.id + "]; \n" +
                "_currentElapsed = ((new Date()) - _currentAnim.startTime); \n" +
                "_currentAnim.progress = _currentElapsed / " + (period*1000) + " % 1; \n" +
                "if (_currentElapsed >= " + (duration*1000) + ") \n" +
                "  _currentAnim.stop(true); \n" +
                "else \n" +
                "  _currentAnim.onTick(); \n";
}

Animator.prototype.onAnimationEnd = null;
Animator.prototype._onAnimationEnd = null;

Animator.prototype.start = function () {
    this.progress = 0;
    this.startTime = new Date();
    this.timer.ticking = true;
}

Animator.prototype.stop = function (finalize) {
    this.timer.ticking = false;
    
    // Default to finalizing the animation when stopping.
    if (typeof finalize == "undefined")
        finalize = true;
        
    if (finalize) {
        if (this._onAnimationEnd)
            this._onAnimationEnd();
        if (this.onAnimationEnd) {
            if (typeof this.onAnimationEnd == "function")
                this.onAnimationEnd();
            else if (typeof this.onAnimationEnd == "string")
                eval(this.onAnimationEnd);
        }
    }
}

