var PENDING_QUIT = false;

//var updateTimeEverySecond = false;
var suspended = 0;
var manualSuspend = 0;
//var prevID = -1;
var updateCounter = 0;

var currentStatus = new iTunesStatus();
var currentTrack = new iTunesTrack();

var FADE_IN_TIME = 750; // ms
var FADE_OUT_TIME = 5000; // ms


/************************* iTunes Voodoo *************************/
		
String.prototype.isPrintableAscii = function () {
    for (var i = 0; i < this.length; i++) {
        var code = this.charCodeAt(i);
        if (code > 255 ||
            (code < 32 && code != 10 && code != 13 && code != 9)) {
            return false;
        }
    }
    return true;
}

function telliTunes(command, timeout) {
    if (system.platform != "macintosh") {
        log("Ignoring command '" + command + "' because telliTunes() (AppleScript) is not supported on Windows.");
        return;
    }
    if (!command.isPrintableAscii()) {
        print("Ignoring telliTunes() command because it contains either non-printing characters or 2+ byte characters:\n" + command);
        return "error";
    }
	return appleScript(	'tell application "iTunes.app" \n' +
	                    '   set my text item delimiters to {return}\n' +
						'	try \n' + 
								command + '\n' +
						'	on error\n' +
						'		return "error" \n' +
						'	end try \n' +
						'end tell \n', timeout ? timeout : 2);
}

function quitiTunes() {
    PENDING_QUIT = true;
    skinElements.trackInfo.data = "Quitting iTunes…";
}

function quitiTunes_internal() {
    PENDING_QUIT = false;
    if (iTunes.running) {
        iTunes.quit();
        skinElements.trackInfo.data = "Waiting for iTunes to quit…";
        while (iTunes.running) {
            log("Waiting for iTunes to quit.");
            sleep(1000);
        }
        update();
    }
}


var suspendDimItems = new Array();

function suspend(message, manual) {
	log('suspend("' + message + '", ' + manual + ');');
	suspended = 1;
	if (manual)
		manualSuspend = 1;
	skinElements.trackInfo.data = message;
	skinElements.trackInfo.inactive = true;
	
	for (var i in suspendDimItems) {
	    if (suspendDimItems[i].inactiveSrc)
	       suspendDimItems[i].src = suspendDimItems[i].inactiveSrc;
 	    else
	       suspendDimItems[i].opacity = 170;
    }
    skinElements.transport_dot.opacity = 0;
    skinElements.pause.opacity = 0;
	currentStatus.clear();
	updateVolIndicators();
	skinElements.ratingStars.opacity = 0;
	artwork.opacity = 0;
	if (system.platform == "windows")
    	iTunesCOMDisconnect();
}

function resume() {
	suspended = 0;
	manualSuspend = 0;
	skinElements.trackInfo.data = '';
	skinElements.trackInfo.inactive = false;
	
    currentTrack.clear();
	
	for (var i in suspendDimItems) {
	   if (suspendDimItems[i].normalSrc)
           suspendDimItems[i].src = suspendDimItems[i].normalSrc;
       suspendDimItems[i].opacity = 255;
    }
	skinElements.pause.opacity = 0;
}

function setVolume(vol) {
	currentStatus.volume = vol;
	if (currentStatus.volume < 0)
		currentStatus.volume = 0;
	else if (currentStatus.volume > 100)
		currentStatus.volume = 100;
    iTunes.volume = currentStatus.volume;
	updateVolIndicators();
}

function changeVolume(delta) {
	setVolume(currentStatus.volume + Number(delta));
}

function setRating(rating) {
	if (rating < 0)
		rating = 0;
	else if (rating > 5)
		rating = 5;
	iTunes.trackRating = currentTrack.rating = rating*20;
	showRating(rating*20);
}


/****************** function update() and friends *******************/
function update() {
	if (manualSuspend == 1)
		return;
		
    currentStatus.update();
    
	if (currentStatus.running) {
        if (suspended == 1) {
            resume();
        }
        var newTrack = false;
        if (currentStatus.player != "stopped" &&
              !(system.platform == "windows" && currentStatus.player == "paused")) {
            newTrack = currentTrack.update();
        }
        updateGUI(newTrack);
    }
	else {
	    if (preferences.autoHide.value == 1) {
	        main_window.visible = 0;
	        artwork_window.visible = 0;
        }
		if (suspended == 0) {
			suspend("iTunes not running.  Click the iTunes button to launch.");
        }
    }
}

function updateGUI(newTrack) {
    if (!main_window.visible)
        main_window.visible = true;
	
    updateVolIndicators();

	// Toggle between play and pause button.
	switch (currentStatus.player) {
		case "stopped":
		case "paused":
			skinElements.play.opacity = 255;
			skinElements.pause.opacity = 0;
			break;
		case "playing":
		case "fast forwarding":
		case "rewinding":
			skinElements.play.opacity = 0;
			skinElements.pause.opacity = 255;
			break;
		default:
		    log("Unknown player status: " + currentStatus.player);
	}
	
	if (currentStatus.player == "stopped")
    {
        currentTrack.clear();
        skinElements.trackInfo.data = "";
        skinElements.trackTime.data = "";
        skinElements.transport_dot.opacity = 0;
        skinElements.transport_frame.opacity = 0;
        showRating(0);
        skinElements.ratingStars.opacity = 0;
        clearArtwork();
		return;
	}
	if (system.platform == "windows" && currentStatus.player == "paused")
	    return;
	        
    if (newTrack) {
        // Activate the "shuttle" if not streaming...
        if (currentTrack.type == "audio stream" || currentTrack.time == 0) {
            skinElements.transport_frame.visible = false;
            skinElements.transport_dot.visible = false;
        }
        else {
            skinElements.transport_frame.visible = true;
            skinElements.transport_dot.visible = true;
        }
                                 
        // Set the rating stars...	
        if (preferences.displayRating.value == 1)
            showRating(iTunes.trackRating ? iTunes.trackRating : 0);
        
        // Set the textual display...
        displayTrackInfo();
        
        // Fetch (potentially asynchronously) the artwork...
        getArtwork();
        
        // Set the track time display...
        setTrackTime();
        
        // Fade it all in...
        fadeTrackIn();
    }
    else if (skinElements.trackInfo.opacity == 0) {
        // When iTunes stays on same track after the track info has faded out already...
        fadeTrackIn();
    }
	
    // Set the track position indicator...
    setTrackTime();
    fadeOutWhenNearingEnd();
}

function fadeOutWhenNearingEnd() {
    if (currentStatus.player == "playing"
            && !skinElements.transport_frame.mouseDown
            && currentStatus.position < currentTrack.time
            && currentTrack.time - currentStatus.position <= 5
            && !trackFadeAnimator.running) {
        fadeTrackOut(1000*(currentTrack.time - currentStatus.position));
    }
}

function fadeTrackIn() {
    trackFadeAnimator.startOpacity = 0;
    trackFadeAnimator.endOpacity = 255;
    trackFadeAnimator.period = FADE_IN_TIME;
    trackFadeAnimator.start();
}

function fadeTrackOut(time) {
    if (typeof time == "undefined")
        time = FADE_OUT_TIME;
    trackFadeAnimator.startOpacity = 255;
    trackFadeAnimator.endOpacity = 0;
    trackFadeAnimator.period = time;
    trackFadeAnimator.start();
}

function displayTrackInfo() {
    // Set all the textual info...
    var newTrackString = "";
    if (preferences.displayArtist.value == 1)
        newTrackString += currentTrack.artist;
        
    if (preferences.displayAlbum.value == 1 &&
        currentTrack.album != "")
    {
        if (newTrackString != "")
            newTrackString += skinElements.trackInfo.separator;
        newTrackString += currentTrack.album;
    }
        
    if (preferences.displayTitle.value == 1 &&
        currentTrack.title != "")
    {
        if (newTrackString != "")
            newTrackString += skinElements.trackInfo.separator;
        newTrackString += '“' + currentTrack.title + '”';
    }
        
    if (preferences.displayTrackNumber.value == 1 &&
        currentTrack.type != 'audio stream')
    {
        if (currentTrack.number != "0") {
            var numString = currentTrack.number;
            if (currentTrack.count != "0") {
                numString += '/' + currentTrack.count;
            }
            newTrackString += ' (' + numString + ')';
        }
    }
    
    skinElements.trackInfo.opacity = 0;
    skinElements.trackInfo.data = newTrackString;
}

function setTrackTime() {
    if (!skinElements.transport_frame.visible) {
        skinElements.trackTime.data = "";
    }
    else if (typeof currentStatus.position == "number") {
        with (skinElements) {
            trackTime.data = getTimeString();
            if (currentTrack.time > 0) {
                transport_dot.hOffset = transport_frame.hOffset + transport_frame.dotOffset + (transport_frame.movementRange * currentStatus.position / currentTrack.time);
            }
        }
    }
}

function getTimeString() {
    if (currentTrack.time == 0)
        return "+" + secondsToTime(currentStatus.position);

    switch (preferences.displayTimeType.value)
    {
      case "Elapsed":
        return "+" + secondsToTime(currentStatus.position);
      case "Remaining":
        return "-" + secondsToTime(currentTrack.time - currentStatus.position);
      case "Off":
      case "Total":
      default:
        return "" + secondsToTime(currentTrack.time);
    }
}

function secondsToTime(s) {
    var mins = Math.floor(s / 60);
    var secs = s % 60;
    return mins + ":" + (secs < 10 ? ("0" + secs) : secs);
}

function decrementTime(force) {
    if (suspended)
        return;
    if (force)
        currentStatus.position = iTunes.playerPosition;
    else if (currentStatus.player == "playing")
        currentStatus.position++;
    if (currentStatus.player == "stopped")
        skinElements.trackTime.data = "";
    else {
        setTrackTime();
        fadeOutWhenNearingEnd();
    }
}

var trackFadeAnimator = new DDFadeAnimator(0, 255, FADE_IN_TIME, false);
trackFadeAnimator.fps = 10;

